@@ -6,14 +6,37 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.06.28 16:00:00                  #
+# Updated Date: 2025.07.10 02:00:00                  #
 # ================================================== #
 
 import os
+import builtins
+import io
 
 # disable warnings
 os.environ["TRANSFORMERS_NO_ADVISORY_WARNINGS"] = "1"
 
+_original_open = builtins.open
+
+def open_wrapper(file, mode='r', *args, **kwargs):
+    """
+    Patch for `builtins.open` - issue #116
+    Prevents attempts to open the .env file in the /home directory
+    within a Snapcraft environment when executing `dotenv.load_dotenv()`.
+    """
+    path = str(file)
+    if ("SNAP" in os.environ
+            and "SNAP_NAME" in os.environ
+            and os.environ["SNAP_NAME"] == "pygpt"):
+        if path == '.env' or path.endswith('/.env'):
+            if 'b' in mode:
+                return io.BytesIO(b"")
+            else:
+                return io.StringIO("")
+    return _original_open(file, mode, *args, **kwargs)
+
+builtins.open = open_wrapper
+
 from pygpt_net.launcher import Launcher
 
 # plugins
@@ -53,6 +76,7 @@ from pygpt_net.provider.llms.google import GoogleLLM
 from pygpt_net.provider.llms.hugging_face import HuggingFaceLLM
 from pygpt_net.provider.llms.hugging_face_api import HuggingFaceApiLLM
 from pygpt_net.provider.llms.local import LocalLLM
+from pygpt_net.provider.llms.mistral import MistralAILLM
 from pygpt_net.provider.llms.ollama import OllamaLLM
 from pygpt_net.provider.llms.openai import OpenAILLM
 from pygpt_net.provider.llms.perplexity import PerplexityLLM
@@ -346,6 +370,7 @@ def run(**kwargs):
     # launcher.add_llm(HuggingFaceLLM())
     launcher.add_llm(HuggingFaceApiLLM())
     launcher.add_llm(LocalLLM())
+    launcher.add_llm(MistralAILLM())
     launcher.add_llm(OllamaLLM())
     launcher.add_llm(DeepseekApiLLM())
     launcher.add_llm(PerplexityLLM())
