@@ -6,9 +6,10 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.14 08:00:00                  #
+# Updated Date: 2025.06.28 16:00:00                  #
 # ================================================== #
 
+import os
 from typing import Optional, Any, Dict
 
 from pygpt_net.core.bridge import BridgeContext
@@ -17,8 +18,10 @@ from pygpt_net.core.events import Event, AppEvent, KernelEvent, RenderEvent
 from pygpt_net.core.types import (
     MODE_AGENT,
     MODE_AGENT_LLAMA,
+    MODE_LLAMA_INDEX,
     MODE_ASSISTANT,
     MODE_IMAGE,
+    MODE_CHAT,
 )
 from pygpt_net.item.ctx import CtxItem
 from pygpt_net.utils import trans
@@ -73,6 +76,32 @@ class Input:
                 self.window.controller.agent.common.display_infinity_loop_confirm()
                 return
 
+            # check ollama model
+            model = self.window.core.config.get('model')
+            if model:
+                model_data = self.window.core.models.get(model)
+                if model_data is not None and model_data.is_ollama():
+                    if (mode == MODE_LLAMA_INDEX or
+                            (mode == MODE_CHAT and not model_data.is_openai_supported() and model_data.is_ollama())):
+                        model_id = model_data.get_ollama_model()
+                        # load ENV vars first
+                        if ('env' in model_data.llama_index
+                                and model_data.llama_index['env'] is not None):
+                            for item in model_data.llama_index['env']:
+                                key = item.get('name', '').strip()
+                                value = item.get('value', '').strip()
+                                os.environ[key] = value
+                        status = self.window.core.models.ollama.check_model(model_id)
+                        is_installed = status.get('is_installed', False)
+                        is_model = status.get('is_model', False)
+                        if not is_installed:
+                            self.window.ui.dialogs.alert(trans("dialog.ollama.not_installed"))
+                            return
+                        if not is_model:
+                            self.window.ui.dialogs.alert(
+                                trans("dialog.ollama.model_not_found").replace("{model}", model_id))
+                            return
+
         # listen for stop command
         if self.generating \
                 and text is not None \
@@ -222,14 +251,19 @@ class Input:
             self.generating = False  # unlock as not generating
             return
 
-        # check API key, show monit if no API key
+        # check OpenAI API key, show monit if no API key
         if mode not in self.window.controller.launcher.no_api_key_allowed:
-            if not self.window.controller.chat.common.check_api_key():
-                self.generating = False
-                self.window.dispatch(KernelEvent(KernelEvent.STATE_ERROR, {
-                    "id": "chat",
-                }))
-                return
+            if not self.window.controller.chat.common.check_api_key(monit=False):
+                model = self.window.core.config.get('model')
+                if model:
+                    model_data = self.window.core.models.get(model)
+                    if model_data is not None and model_data.is_gpt():
+                        self.window.controller.chat.common.check_api_key(monit=True)
+                        self.generating = False
+                        self.window.dispatch(KernelEvent(KernelEvent.STATE_ERROR, {
+                            "id": "chat",
+                        }))
+                        return
 
         # set state to: busy
         self.window.dispatch(KernelEvent(KernelEvent.STATE_BUSY, {
@@ -250,6 +284,9 @@ class Input:
             # check if current ctx is allowed for this mode - if not, then auto-create new ctx
             self.window.controller.ctx.handle_allowed(mode)
 
+        # update mode in ctx
+        self.window.controller.ctx.update_mode_in_current()
+
         # send input to API
         if mode == MODE_IMAGE:
             self.window.controller.chat.image.send(
