@@ -6,15 +6,19 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.07 23:00:00                  #
+# Updated Date: 2024.12.14 08:00:00                  #
 # ================================================== #
 
+from typing import Optional, List
+
 from PySide6.QtCore import Slot, QTimer
 
+from pygpt_net.core.events import RenderEvent
 from pygpt_net.core.render.base import BaseRenderer
 from pygpt_net.core.render.markdown.renderer import Renderer as MarkdownRenderer
 from pygpt_net.core.render.plain.renderer import Renderer as PlainTextRenderer
 from pygpt_net.core.render.web.renderer import Renderer as WebRenderer
+from pygpt_net.core.tabs.tab import Tab
 from pygpt_net.core.text.utils import output_html2text, output_clean_html
 from pygpt_net.item.ctx import CtxItem, CtxMeta
 
@@ -43,59 +47,117 @@ class Render:
         self.plaintext_renderer.prepare()
         self.web_renderer.prepare()
 
-    def get_engine(self) -> str:
+    def handle(self, event: RenderEvent):
         """
-        Get current render engine
+        Handle render event
 
-        :return: engine name
+        :param event: RenderEvent
         """
-        return self.engine
+        name = event.name
+        data = event.data
+        if data is None:
+            data = {}
 
-    def get_renderer(self) -> BaseRenderer:
-        """
-        Get current renderer instance
+        chunk = data.get("chunk", "")
+        begin = data.get("begin", False)
+        stream = data.get("stream", False)
+        ctx = data.get("ctx")
+        meta = data.get("meta")
+        tool_data = data.get("tool_data")
+        items = data.get("items")
+        clear = data.get("clear", False)
+        flush = data.get("flush", False)
+        append = data.get("append", False)
+        footer = data.get("footer", False)
+        initialized = data.get("initialized", False)
+        tab = data.get("tab")
 
-        :return: Renderer instance
-        """
-        # get selected renderer
-        if self.window.core.config.get('render.plain'):
-            return self.plaintext_renderer
-        else:
-            if self.engine == "web":
-                return self.web_renderer
-            else:
-                return self.markdown_renderer
+        if name == RenderEvent.BEGIN:
+            self.begin(meta, ctx, stream)
+        elif name == RenderEvent.END:
+            self.end(meta, ctx, stream)
+        elif name == RenderEvent.RELOAD:
+            self.reload()
+        elif name == RenderEvent.RESET:
+            self.reset(meta)
+        elif name == RenderEvent.PREPARE:
+            self.prepare()
 
-    def switch(self, live: bool = True):
-        """
-        Switch renderer (markdown <==> plain text) - active, TODO: remove from settings, leave only checkbox
+        elif name == RenderEvent.STREAM_BEGIN:
+            self.stream_begin(meta, ctx)
+        elif name == RenderEvent.STREAM_APPEND:
+            self.append_chunk(meta, ctx, chunk, begin)
+        elif name == RenderEvent.STREAM_END:
+            self.stream_end(meta, ctx)
 
-        :param live: True if live update
-        """
-        plain = self.window.core.config.get('render.plain')
-        if plain:
-            self.window.controller.theme.markdown.clear()
-            for pid in self.window.ui.nodes['output_plain']:
-                if self.window.ui.nodes['output'][pid] is not None:
-                    self.window.ui.nodes['output'][pid].setVisible(False)
-                    self.window.ui.nodes['output_plain'][pid].setVisible(True)
-        else:
-            self.window.controller.ctx.refresh()  # TODO: move to on_switch
-            self.window.controller.theme.markdown.update(force=True)
-            for pid in self.window.ui.nodes['output']:
-                if self.window.ui.nodes['output'][pid] is not None:
-                    self.window.ui.nodes['output'][pid].setVisible(True)
-                    self.window.ui.nodes['output_plain'][pid].setVisible(False)
+        elif name == RenderEvent.ON_PAGE_LOAD:
+            self.on_page_loaded(meta, tab)
+        elif name ==RenderEvent.ON_THEME_CHANGE:
+            self.on_theme_change()
+        elif name == RenderEvent.ON_LOAD:
+            self.on_load(meta)
+        elif name == RenderEvent.ON_TS_ENABLE:
+            self.on_enable_timestamp(live=initialized)
+        elif name == RenderEvent.ON_TS_DISABLE:
+            self.on_disable_timestamp(live=initialized)
+        elif name == RenderEvent.ON_EDIT_ENABLE:
+            self.on_enable_edit(live=initialized)
+        elif name == RenderEvent.ON_EDIT_DISABLE:
+            self.on_disable_edit(live=initialized)
+        elif name == RenderEvent.ON_SWITCH:
+            self.switch()
 
-    def get_pid(self, meta: CtxMeta):
+        elif name == RenderEvent.CLEAR_INPUT:
+            self.clear_input()
+        elif name == RenderEvent.CLEAR_OUTPUT:
+            self.clear_output(meta)
+        elif name == RenderEvent.CLEAR_ALL:
+            self.clear_all()
+        elif name == RenderEvent.CLEAR:
+            self.clear(meta)
+
+        elif name == RenderEvent.TOOL_UPDATE:
+            self.tool_output_update(meta, tool_data)
+        elif name == RenderEvent.TOOL_CLEAR:
+            self.tool_output_clear(meta)
+        elif name == RenderEvent.TOOL_BEGIN:
+            self.tool_output_begin(meta)
+        elif name == RenderEvent.TOOL_END:
+            self.tool_output_end()
+
+        elif name == RenderEvent.CTX_APPEND:
+            self.append_context(meta, items, clear)
+        elif name == RenderEvent.INPUT_APPEND:
+            self.append_input(meta, ctx, flush, append)
+        elif name == RenderEvent.OUTPUT_APPEND:
+            self.append_output(meta, ctx)
+
+        elif name == RenderEvent.EXTRA_APPEND:
+            self.append_extra(meta, ctx, footer)
+        elif name == RenderEvent.EXTRA_END:
+            self.end_extra(meta, ctx)
+
+        elif name == RenderEvent.ACTION_REGEN_SUBMIT:
+            self.on_reply_submit(ctx)
+        elif name == RenderEvent.ACTION_EDIT_SUBMIT:
+            self.on_edit_submit(ctx)
+
+
+    def get_pid(self, meta: CtxMeta) -> int:
         """
         Get PID for context meta
 
-        :param meta: context PID
+        :param meta: context meta
+        :return: PID
         """
-        self.get_renderer().get_pid(meta)
+        return self.instance().get_pid(meta)
 
-    def begin(self, meta: CtxMeta, ctx: CtxItem, stream: bool = False):
+    def begin(
+            self,
+            meta: CtxMeta,
+            ctx: CtxItem,
+            stream: bool = False
+    ):
         """
         Render begin
 
@@ -103,10 +165,15 @@ class Render:
         :param ctx: context item
         :param stream: True if it is a stream
         """
-        self.get_renderer().begin(meta, ctx, stream)
+        self.instance().begin(meta, ctx, stream)
         self.update()
 
-    def end(self, meta: CtxMeta, ctx: CtxItem, stream: bool = False):
+    def end(
+            self,
+            meta: CtxMeta,
+            ctx: CtxItem,
+            stream: bool = False
+    ):
         """
         Render end
 
@@ -114,10 +181,15 @@ class Render:
         :param ctx: context item
         :param stream: True if it is a stream
         """
-        self.get_renderer().end(meta, ctx, stream)
+        self.instance().end(meta, ctx, stream)
         self.update()
 
-    def end_extra(self, meta: CtxMeta, ctx: CtxItem, stream: bool = False):
+    def end_extra(
+            self,
+            meta: CtxMeta,
+            ctx: CtxItem,
+            stream: bool = False
+    ):
         """
         Render end extra
         
@@ -125,78 +197,106 @@ class Render:
         :param ctx: context item
         :param stream: True if it is a stream
         """
-        self.get_renderer().end_extra(meta, ctx, stream)
+        self.instance().end_extra(meta, ctx, stream)
         self.update()
 
-    def stream_begin(self, meta: CtxMeta, ctx: CtxItem):
+    def stream_begin(
+            self,
+            meta: CtxMeta,
+            ctx: CtxItem
+    ):
         """
         Render stream begin
         
         :param meta: context meta
         :param ctx: context item
         """
-        self.get_renderer().stream_begin(meta, ctx)
+        self.instance().stream_begin(meta, ctx)
         self.update()
 
-    def stream_end(self, meta: CtxMeta, ctx: CtxItem):
+    def stream_end(
+            self,
+            meta: CtxMeta,
+            ctx: CtxItem
+    ):
         """
         Render stream end
         
         :param meta: context meta
         :param ctx: context item
         """
-        self.get_renderer().stream_end(meta, ctx)
+        self.instance().stream_end(meta, ctx)
         self.update()
 
-    def clear_output(self, meta: CtxMeta = None):
+    def clear_output(
+            self,
+            meta: Optional[CtxMeta] = None
+    ):
         """
         Clear current active output
 
-        :param meta: Context meta
+        :param meta: context meta
         """
-        self.get_renderer().clear_output(meta) # TODO: get meta id on load
+        self.instance().clear_output(meta) # TODO: get meta id on load
         self.update()
 
     def clear_input(self):
         """Clear input"""
-        self.get_renderer().clear_input()
+        self.instance().clear_input()
 
-    def on_load(self, meta: CtxMeta = None):
+    def on_load(
+            self,
+            meta: Optional[CtxMeta] = None
+    ):
         """
         On load (meta)
 
-        :param meta: Context meta
+        :param meta: context meta
         """
-        self.get_renderer().on_load(meta)
+        self.instance().on_load(meta)
         self.update()
         self.window.controller.ui.tabs.update_tooltip(meta.name)  # update tab tooltip
 
-    def reset(self, meta: CtxMeta = None):
+    def reset(
+            self,
+            meta: Optional[CtxMeta] = None
+    ):
         """
         Reset current meta
 
         :param meta: Context meta
         """
-        self.get_renderer().reset(meta)  # TODO: get meta id on load
+        self.instance().reset(meta)  # TODO: get meta id on load
         self.update()
 
     def reload(self):
         """Reload current output"""
-        self.get_renderer().reload()  # TODO: or all outputs?
+        self.instance().reload()  # TODO: or all outputs?
         self.update()
 
-    def append_context(self, meta: CtxMeta, items: list, clear: bool = True):
+    def append_context(
+            self,
+            meta: CtxMeta,
+            items: List[CtxItem],
+            clear: bool = True
+    ):
         """
         Append all context to output
 
-        :param meta: Context meta
+        :param meta: context meta
         :param items: context items
         :param clear: True if clear all output before append
         """
-        self.get_renderer().append_context(meta, items, clear)
+        self.instance().append_context(meta, items, clear)
         self.update()
 
-    def append_input(self, meta: CtxMeta, ctx: CtxItem, flush: bool = True, append: bool = False):
+    def append_input(
+            self,
+            meta: CtxMeta,
+            ctx: CtxItem,
+            flush: bool = True,
+            append: bool = False
+    ):
         """
         Append text input to output
         
@@ -205,20 +305,29 @@ class Render:
         :param flush: True if flush output
         :param append: True to force append node
         """
-        self.get_renderer().append_input(meta, ctx, flush=flush, append=append)
+        self.instance().append_input(meta, ctx, flush=flush, append=append)
         self.update()
 
-    def append_output(self, meta: CtxMeta, ctx: CtxItem):
+    def append_output(
+            self,
+            meta: CtxMeta,
+            ctx: CtxItem
+    ):
         """
         Append text output to output
         
         :param meta: context meta
         :param ctx: context item
         """
-        self.get_renderer().append_output(meta, ctx)
+        self.instance().append_output(meta, ctx)
         self.update()
 
-    def append_extra(self, meta: CtxMeta, ctx: CtxItem, footer: bool = False):
+    def append_extra(
+            self,
+            meta: CtxMeta,
+            ctx: CtxItem,
+            footer: bool = False
+    ):
         """
         Append extra data (images, files, etc.) to output
         
@@ -226,10 +335,16 @@ class Render:
         :param ctx: context item
         :param footer: True if it is a footer
         """
-        self.get_renderer().append_extra(meta, ctx, footer)
+        self.instance().append_extra(meta, ctx, footer)
         self.update()
 
-    def append_chunk(self, meta: CtxMeta, ctx: CtxItem, text_chunk: str, begin: bool = False):
+    def append_chunk(
+            self,
+            meta: CtxMeta,
+            ctx: CtxItem,
+            text_chunk: str,
+            begin: bool = False
+    ):
         """
         Append output stream chunk to output
         
@@ -238,7 +353,7 @@ class Render:
         :param text_chunk: text chunk
         :param begin: if it is the beginning of the stream
         """
-        self.get_renderer().append_chunk(meta, ctx, text_chunk, begin)
+        self.instance().append_chunk(meta, ctx, text_chunk, begin)
         self.update()
 
     def on_enable_edit(self, live: bool = True):
@@ -247,7 +362,7 @@ class Render:
 
         :param live: True if live update
         """
-        self.get_renderer().on_enable_edit(live)
+        self.instance().on_enable_edit(live)
         self.update()
 
     def on_disable_edit(self, live: bool = True):
@@ -256,7 +371,7 @@ class Render:
 
         :param live: True if live update
         """
-        self.get_renderer().on_disable_edit(live)
+        self.instance().on_disable_edit(live)
         self.update()
 
     def on_enable_timestamp(self, live: bool = True):
@@ -265,7 +380,7 @@ class Render:
 
         :param live: True if live update
         """
-        self.get_renderer().on_enable_timestamp(live)
+        self.instance().on_enable_timestamp(live)
         self.update()
 
     def on_disable_timestamp(self, live: bool = True):
@@ -274,7 +389,7 @@ class Render:
 
         :param live: True if live update
         """
-        self.get_renderer().on_disable_timestamp(live)
+        self.instance().on_disable_timestamp(live)
         self.update()
 
     def remove_item(self, id: int):
@@ -283,7 +398,7 @@ class Render:
 
         :param id: context item ID
         """
-        self.get_renderer().remove_item(id)
+        self.instance().remove_item(id)
         self.update()
 
     def remove_items_from(self, id: int):
@@ -292,7 +407,7 @@ class Render:
 
         :param id: context item ID
         """
-        self.get_renderer().remove_items_from(id)
+        self.instance().remove_items_from(id)
         self.update()
 
     def on_edit_submit(self, ctx: CtxItem):
@@ -301,7 +416,7 @@ class Render:
 
         :param ctx: context item
         """
-        self.get_renderer().on_edit_submit(ctx)
+        self.instance().on_edit_submit(ctx)
         self.update()
 
     def on_remove_submit(self, ctx: CtxItem):
@@ -310,7 +425,7 @@ class Render:
 
         :param ctx: context item
         """
-        self.get_renderer().on_remove_submit(ctx)
+        self.instance().on_remove_submit(ctx)
         self.update()
 
     def on_reply_submit(self, ctx: CtxItem):
@@ -319,16 +434,21 @@ class Render:
 
         :param ctx: context item
         """
-        self.get_renderer().on_reply_submit(ctx)
+        self.instance().on_reply_submit(ctx)
         self.update()
 
-    def on_page_loaded(self, meta: CtxMeta):
+    def on_page_loaded(
+            self,
+            meta: CtxMeta,
+            tab: Optional[Tab] = None
+    ):
         """
         On page loaded callback
 
-        :param meta: context item
+        :param meta: context meta
+        :param tab: Tab
         """
-        self.get_renderer().on_page_loaded(meta)  # TODO: send ID with callback
+        self.instance().on_page_loaded(meta, tab)
         self.update()
 
     def on_theme_change(self):
@@ -345,7 +465,7 @@ class Render:
 
         :return: scroll position
         """
-        return self.get_renderer().get_scroll_position()
+        return self.instance().get_scroll_position()
 
     def set_scroll_position(self, position: int):
         """
@@ -353,7 +473,7 @@ class Render:
 
         :param position: scroll position
         """
-        self.get_renderer().set_scroll_position(position)
+        self.instance().set_scroll_position(position)
         self.update()
 
     def update(self):
@@ -367,32 +487,40 @@ class Render:
 
         :param meta: ctx meta instance
         """
-        self.window.controller.chat.render.reset(meta)
-        self.window.controller.chat.render.clear_output(meta)
+        self.reset(meta)
+        self.clear_output(meta)
 
     def clear_all(self):
         """Clear all"""
-        self.get_renderer().clear_all()
+        self.instance().clear_all()
         self.update()
 
-    def tool_output_append(self, meta: CtxMeta, content: str):
+    def tool_output_append(
+            self,
+            meta: CtxMeta,
+            content: str
+    ):
         """
         Add tool output (append)
 
         :param meta: context meta
         :param content: content
         """
-        self.get_renderer().tool_output_append(meta, content)
+        self.instance().tool_output_append(meta, content)
         self.update()
 
-    def tool_output_update(self, meta: CtxMeta, content: str):
+    def tool_output_update(
+            self,
+            meta: CtxMeta,
+            content: str
+    ):
         """
         Replace tool output
 
         :param meta: context meta
         :param content: content
         """
-        self.get_renderer().tool_output_update(meta, content)
+        self.instance().tool_output_update(meta, content)
         self.update()
 
     def tool_output_clear(self, meta: CtxMeta):
@@ -401,7 +529,7 @@ class Render:
 
         :param meta: context meta
         """
-        self.get_renderer().tool_output_clear(meta)
+        self.instance().tool_output_clear(meta)
         self.update()
 
     def tool_output_begin(self, meta: CtxMeta):
@@ -410,7 +538,7 @@ class Render:
 
         :param meta: context meta
         """
-        self.get_renderer().tool_output_begin(meta)
+        self.instance().tool_output_begin(meta)
         self.update()
 
     def tool_output_end(self):
@@ -419,11 +547,57 @@ class Render:
 
         :param meta: context meta
         """
-        self.get_renderer().tool_output_end()
+        self.instance().tool_output_end()
         self.update()
 
+    def get_engine(self) -> str:
+        """
+        Get current render engine ID
+
+        :return: engine name
+        """
+        return self.engine
+
+    def switch(self):
+        """
+        Switch renderer (markdown/web <==> plain text) - active, TODO: remove from settings, leave only checkbox
+        """
+        plain = self.window.core.config.get('render.plain')
+        if plain:
+            self.window.controller.theme.markdown.clear()
+            for pid in self.window.ui.nodes['output_plain']:
+                if self.window.ui.nodes['output_plain'][pid] is not None:
+                    self.window.ui.nodes['output'][pid].setVisible(False)
+                    self.window.ui.nodes['output_plain'][pid].setVisible(True)
+        else:
+            self.window.controller.ctx.refresh()  # TODO: move to on_switch
+            self.window.controller.theme.markdown.update(force=True)
+            for pid in self.window.ui.nodes['output']:
+                if self.window.ui.nodes['output'][pid] is not None:
+                    self.window.ui.nodes['output'][pid].setVisible(True)
+                    self.window.ui.nodes['output_plain'][pid].setVisible(False)
+
+    def instance(self) -> BaseRenderer:
+        """
+        Get instance of current renderer
+
+        :return: Renderer instance
+        """
+        # get selected renderer
+        if self.window.core.config.get('render.plain'):
+            return self.plaintext_renderer
+        else:
+            if self.engine == "web":
+                return self.web_renderer
+            else:
+                return self.markdown_renderer
+
     @Slot(str, str)
-    def handle_save_as(self, text: str, type: str = 'txt'):
+    def handle_save_as(
+            self,
+            text: str,
+            type: str = 'txt'
+    ):
         """
         Handle save as signal  # TODO: move to another class
 
