@@ -6,12 +6,13 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.14 08:00:00                  #
+# Updated Date: 2025.07.08 02:00:00                  #
 # ================================================== #
 
 import copy
-from typing import Optional
+from typing import Optional, Any
 
+from pygpt_net.core.events import Event
 from pygpt_net.utils import trans
 
 
@@ -26,6 +27,7 @@ class Editor:
         self.dialog = False
         self.config_initialized = False
         self.current = None
+        self.previous = None  # previous models
         self.width = 800
         self.height = 500
         self.options = {
@@ -37,6 +39,11 @@ class Editor:
                 "type": "text",
                 "label": "model.name",
             },
+            "provider": {
+                "type": "combo",
+                "use": "llm_providers",
+                "label": "model.provider",
+            },
             "ctx": {
                 "type": "int",
                 "label": "model.ctx",
@@ -48,60 +55,26 @@ class Editor:
                 "description": "model.tokens.desc",
             },
             "mode": {
-                "type": "text",  # list of comma separated values
+                "type": "bool_list",  # list of comma separated values
                 "label": "model.mode",
-                "description": "model.mode.desc",
+                "use": "modes",
+            },
+            "input": {
+                "type": "bool_list",  # list of comma separated values
+                "label": "model.input",
+                "use": "multimodal",
+                "advanced": True,
+            },
+            "output": {
+                "type": "bool_list",  # list of comma separated values
+                "label": "model.output",
+                "use": "multimodal",
+                "advanced": True,
             },
             "default": {
                 "type": "bool",
                 "label": "model.default",
             },
-            "langchain.provider": {
-                "type": "combo",
-                "use": "langchain_providers",
-                "label": "model.langchain.provider",
-                "description": "model.langchain.provider.desc",
-            },
-            "langchain.mode": {
-                "type": "text",  # list of comma separated values
-                "label": "model.langchain.mode",
-                "description": "model.langchain.mode.desc",
-            },
-            "langchain.args": {
-                "type": "dict",
-                "keys": {
-                    'name': 'text',
-                    'value': 'text',
-                    'type': {
-                        "type": "combo",
-                        "use": "var_types",
-                    },
-                },
-                "label": "model.langchain.args",
-                "description": "model.langchain.args.desc",
-                "advanced": True,
-            },
-            "langchain.env": {
-                "type": "dict",
-                "keys": {
-                    'name': 'text',
-                    'value': 'text',
-                },
-                "label": "model.langchain.env",
-                "description": "model.langchain.env.desc",
-                "advanced": True,
-            },
-            "llama_index.provider": {
-                "type": "combo",
-                "use": "llama_index_providers",
-                "label": "model.llama_index.provider",
-                "description": "model.llama_index.provider.desc",
-            },
-            "llama_index.mode": {
-                "type": "text",  # list of comma separated values
-                "label": "model.llama_index.mode",
-                "description": "model.llama_index.mode.desc",
-            },
             "llama_index.args": {
                 "type": "dict",
                 "keys": {
@@ -150,17 +123,35 @@ class Editor:
         """Set up editor"""
         idx = None
         self.window.model_settings.setup(idx)  # widget dialog setup
-        parent = "model"
-        keys = [
-            "langchain.args",
-            "langchain.env",
-        ]
-        for key in keys:
-            self.window.ui.dialogs.register_dictionary(
-                key,
-                parent,
-                self.get_option(key),
-            )
+        self.window.ui.add_hook("update.model.name", self.hook_update)
+        self.window.ui.add_hook("update.model.mode", self.hook_update)
+
+    def hook_update(
+            self,
+            key: str,
+            value: Any,
+            caller,
+            *args,
+            **kwargs
+    ):
+        """
+        Hook: on settings update
+
+        :param key: config key
+        :param value: config value
+        :param caller: caller name
+        :param args: args
+        :param kwargs: kwargs
+        """
+        if self.window.controller.reloading:
+            return  # ignore hooks during reloading process
+        if key in ["id", "name", "mode"]:
+            self.save(persist=False)
+            self.reload_items()
+            # select by current model
+            idx = self.get_tab_by_id(self.current)
+            if idx is not None:
+                self.set_by_tab(idx)
 
     def toggle_editor(self):
         """Toggle models editor dialog"""
@@ -180,6 +171,7 @@ class Editor:
             self.config_initialized = True
         if not self.dialog or force:
             self.init()
+            self.previous =  copy.deepcopy(self.window.core.models.items)
             self.window.ui.dialogs.open(
                 "models.editor",
                 width=self.width,
@@ -187,6 +179,14 @@ class Editor:
             )
             self.dialog = True
 
+    def undo(self):
+        """Undo last changes in models editor"""
+        if self.previous is not None:
+            self.window.core.models.items = copy.deepcopy(self.previous)
+            self.window.core.models.save()
+            self.reload_items()
+            self.init()
+
     def close(self):
         """Close models editor dialog"""
         if self.dialog:
@@ -195,6 +195,7 @@ class Editor:
 
     def init(self):
         """Initialize models editor options"""
+        self.window.core.models.sort_items()
         self.reload_items()
 
         # select the first plugin on list if no plugin selected yet
@@ -204,13 +205,14 @@ class Editor:
 
         # assign plugins options to config dialog fields
         options = copy.deepcopy(self.get_options())  # copy options
-        model = self.window.core.models.items[self.current]
-        data_dict = model.to_dict()
-        for key in options:
-            value = data_dict[key]
-            options[key]["value"] = value
+        if self.current in self.window.core.models.items:
+            model = self.window.core.models.items[self.current]
+            data_dict = model.to_dict()
+            for key in options:
+                value = data_dict[key]
+                options[key]["value"] = value
 
-        if self.current is not None:
+        if self.current is not None and self.current in self.window.core.models.items:
             self.set_tab_by_id(self.current)
 
         # load and apply options to config dialog
@@ -231,7 +233,18 @@ class Editor:
                 option=options[key],
             )
             data_dict[key] = value
-        self.window.core.models.items[self.current].from_dict(data_dict)
+
+        # update current model
+        if self.current in self.window.core.models.items:
+            self.window.core.models.items[self.current].from_dict(data_dict)
+            if persist:
+                # change key to model ID if key not exists
+                old_key = self.current
+                new_key = self.window.core.models.items[self.current].id
+                if old_key != new_key:
+                    if new_key not in self.window.core.models.items:
+                        self.window.core.models.items[new_key] = self.window.core.models.items.pop(old_key)
+                        self.current = new_key  # switch current key
 
         # save config
         if persist:
@@ -239,6 +252,13 @@ class Editor:
             self.close()
             self.window.update_status(trans("info.settings.saved"))
 
+        self.window.core.models.sort_items()
+        self.window.controller.model.reload()
+
+        # dispatch on update event
+        event = Event(Event.MODELS_CHANGED)
+        self.window.dispatch(event, all=True)
+
     def reload_items(self):
         """Reload items"""
         items = self.window.core.models.items
@@ -302,12 +322,7 @@ class Editor:
             )
             return
 
-        # reload settings window
-        self.window.core.models.save()
-        self.current = None
-        self.reload_items()
-        self.init()
-        # self.window.ui.dialogs.alert(trans('dialog.models.editor.defaults.user.result'))
+        self.undo()
 
     def load_defaults_app(self, force: bool = False):
         """
@@ -329,11 +344,13 @@ class Editor:
 
         # reload settings window
         self.current = None
+        self.window.core.models.sort_items()
         self.reload_items()
         self.init()
         self.window.ui.dialogs.alert(
             trans('dialog.models.editor.defaults.app.result')
         )
+        self.previous = copy.deepcopy(self.window.core.models.items)
 
     def set_by_tab(self, idx: int):
         """
