@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.14 08:00:00                  #
+# Updated Date: 2025.06.28 16:00:00                  #
 # ================================================== #
 
 import datetime
@@ -25,6 +25,7 @@ from pygpt_net.core.types import (
     MODE_LLAMA_INDEX,
     MODE_VISION, 
     MODE_IMAGE,
+    MODE_RESEARCH,
 )
 from pygpt_net.item.preset import PresetItem
 from pygpt_net.utils import trans
@@ -73,10 +74,10 @@ class Editor:
                 "type": "bool",
                 "label": "preset.vision",
             },
-            MODE_LANGCHAIN: {
-                "type": "bool",
-                "label": "preset.langchain",
-            },
+            #MODE_LANGCHAIN: {
+            #    "type": "bool",
+            #    "label": "preset.langchain",
+            #},
             MODE_EXPERT: {
                 "type": "bool",
                 "label": "preset.expert",
@@ -93,6 +94,10 @@ class Editor:
                 "type": "bool",
                 "label": "preset.audio",
             },
+            MODE_RESEARCH: {
+                "type": "bool",
+                "label": "preset.research",
+            },
             # "assistant": {
             # "type": "bool",
             # "label": "preset.assistant",
@@ -288,8 +293,8 @@ class Editor:
                 data.img = True
             elif mode == MODE_VISION:
                 data.vision = True
-            elif mode == MODE_LANGCHAIN:
-                data.langchain = True
+            # elif mode == MODE_LANGCHAIN:
+                # data.langchain = True
             # elif mode == MODE_ASSISTANT:
                 # data.assistant = True
             elif mode == MODE_LLAMA_INDEX:
@@ -302,6 +307,8 @@ class Editor:
                 data.agent_llama = True
             elif mode == MODE_AUDIO:
                 data.audio = True
+            elif mode == MODE_RESEARCH:
+                data.research = True
 
         options = {}
         data_dict = data.to_dict()
@@ -366,7 +373,7 @@ class Editor:
             MODE_COMPLETION,
             MODE_IMAGE,
             MODE_VISION,
-            MODE_LANGCHAIN,
+            # MODE_LANGCHAIN,
             MODE_LLAMA_INDEX,
             MODE_EXPERT,
             MODE_AGENT_LLAMA,
