@@ -6,12 +6,13 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.14 08:00:00                  #
+# Updated Date: 2025.06.30 02:00:00                  #
 # ================================================== #
 
 import copy
 from typing import Optional, Any, Dict
 
+from pygpt_net.core.events import Event
 from pygpt_net.utils import trans
 
 
@@ -203,9 +204,16 @@ class Editor:
         if self.config_changed('access.shortcuts'):
             self.window.setup_global_shortcuts()
 
+        # update ENV
+        self.window.core.config.setup_env()
+
         self.before_config = copy.deepcopy(self.window.core.config.all())
         self.window.controller.settings.close_window(id)
 
+        # dispatch on update event
+        event = Event(Event.SETTINGS_CHANGED)
+        self.window.dispatch(event, all=True)
+
     def config_changed(self, key: str) -> bool:
         """
         Check if config changed
