@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.26 19:00:00                  #
+# Updated Date: 2025.03.02 19:00:00                  #
 # ================================================== #
 
 import json
@@ -31,6 +31,7 @@ class PresetItem:
         self.agent_llama = False
         self.expert = False
         self.audio = False
+        self.research = False
         self.temperature = 1.0
         self.filename = None
         self.model = None
@@ -65,6 +66,7 @@ class PresetItem:
             "agent_llama": self.agent_llama,
             "expert": self.expert,
             "audio": self.audio,
+            "research": self.research,
             "temperature": self.temperature,
             "filename": self.filename,
             "model": self.model,
@@ -109,6 +111,8 @@ class PresetItem:
             self.expert = data["expert"]
         if "audio" in data:
             self.audio = data["audio"]
+        if "research" in data:
+            self.research = data["research"]
         if "temperature" in data:
             self.temperature = data["temperature"]
         if "filename" in data:
