@@ -6,9 +6,11 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.01.06 04:00:00                  #
+# Updated Date: 2024.12.14 22:00:00                  #
 # ================================================== #
 
+from typing import Dict
+
 from packaging.version import Version
 
 from pygpt_net.item.calendar_note import CalendarNoteItem
@@ -35,13 +37,13 @@ class BaseProvider:
     def load(self, year, month, day) -> CalendarNoteItem:
         pass
 
-    def load_all(self) -> dict:
+    def load_all(self) -> Dict[str, CalendarNoteItem]:
         pass
 
     def save(self, notepad: CalendarNoteItem):
         pass
 
-    def save_all(self, items: dict):
+    def save_all(self, items: Dict[str, CalendarNoteItem]):
         pass
 
     def remove(self, year, month, day):
