@@ -6,13 +6,15 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.03.06 02:00:00                  #
+# Updated Date: 2024.12.14 22:00:00                  #
 # ================================================== #
 
+from typing import Tuple, Dict, Any
+
 from pygpt_net.utils import unpack_var
 
 
-def unpack_file_item(row: dict) -> (str, dict):
+def unpack_file_item(row: Dict[str, Any]) -> Tuple[str, dict]:
     """
     Unpack item from DB row
 
