@@ -6,8 +6,9 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.26 19:00:00                  #
+# Updated Date: 2025.07.01 01:00:00                  #
 # ================================================== #
+import base64
 
 from httpx_socks import SyncProxyTransport
 
@@ -20,6 +21,7 @@ from pygpt_net.core.types import (
     MODE_COMPLETION,
     MODE_IMAGE,
     MODE_VISION,
+    MODE_RESEARCH,
 )
 from pygpt_net.core.bridge.context import BridgeContext
 
@@ -28,9 +30,11 @@ from .assistants import Assistants
 from .chat import Chat
 from .completion import Completion
 from .image import Image
+from .responses import Responses
 from .store import Store
 from .summarizer import Summarizer
 from .vision import Vision
+from pygpt_net.item.model import ModelItem
 
 
 class Gpt:
@@ -46,14 +50,21 @@ class Gpt:
         self.chat = Chat(window)
         self.completion = Completion(window)
         self.image = Image(window)
+        self.responses = Responses(window)
         self.store = Store(window)
         self.summarizer = Summarizer(window)
         self.vision = Vision(window)
 
-    def get_client(self) -> OpenAI:
+    def get_client(
+            self,
+            mode: str = MODE_CHAT,
+            model: ModelItem = None
+    ) -> OpenAI:
         """
         Return OpenAI client
 
+        :param mode: Mode
+        :param model: Model
         :return: OpenAI client
         """
         args = {
@@ -73,6 +84,9 @@ class Gpt:
                 args["http_client"] = DefaultHttpxClient(
                     transport=transport,
                 )
+
+        # update client args by mode and model
+        args = self.window.core.models.prepare_client_args(args, mode, model)
         return OpenAI(**args)
 
     def call(self, context: BridgeContext, extra: dict = None) -> bool:
@@ -96,6 +110,9 @@ class Gpt:
         ai_name = ctx.output_name
         thread_id = ctx.thread  # from ctx
 
+        # --- Responses API ---- /beta/
+        use_responses_api = False
+
         # get model id
         model_id = None
         if model is not None:
@@ -103,6 +120,13 @@ class Gpt:
             if max_tokens > model.tokens:  # check max output tokens
                 max_tokens = model.tokens
 
+            if model.is_gpt():
+                if (mode in [MODE_CHAT, MODE_RESEARCH]
+                        and self.window.core.config.get('api_use_responses', False)):
+                    use_responses_api = True  # use responses API for chat mode, only OpenAI models
+
+        ctx.use_responses_api = use_responses_api  # set in context
+
         response = None
         used_tokens = 0
         context.max_tokens = max_tokens  # update max output tokens
@@ -116,13 +140,30 @@ class Gpt:
             )
             used_tokens = self.completion.get_used_tokens()
 
-        # chat
-        elif mode in [MODE_CHAT, MODE_AUDIO]:
-            response = self.chat.send(
-                context=context,
-                extra=extra,
-            )
-            used_tokens = self.chat.get_used_tokens()
+        # chat, audio (OpenAI) | research (Perplexity)
+        elif mode in [
+            MODE_CHAT,
+            MODE_AUDIO,
+            MODE_RESEARCH
+        ]:
+            # responses API
+            if use_responses_api:
+                response = self.responses.send(
+                    context=context,
+                    extra=extra,
+                )
+                used_tokens = self.responses.get_used_tokens()
+            else:
+                # chat completion API
+                response = self.chat.send(
+                    context=context,
+                    extra=extra,
+                )
+                if hasattr(response, "citations"):
+                    if response.citations:
+                        ctx.urls = response.citations
+                used_tokens = self.chat.get_used_tokens()
+
             self.vision.append_images(ctx)  # append images to ctx if provided
 
         # image
@@ -165,7 +206,7 @@ class Gpt:
 
         # if stream
         if stream:
-            ctx.stream = response
+            ctx.stream = response  # generator
             ctx.set_output("", ai_name)  # set empty output
             ctx.input_tokens = used_tokens  # get from input tokens calculation
             return True
@@ -182,14 +223,27 @@ class Gpt:
         output = ""
         if mode == MODE_COMPLETION:
             output = response.choices[0].text.strip()
-        elif mode in [MODE_CHAT, MODE_VISION]:
-            if response.choices[0]:
-                if response.choices[0].message.content:
-                    output = response.choices[0].message.content.strip()
-                elif response.choices[0].message.tool_calls:
-                    ctx.tool_calls = self.window.core.command.unpack_tool_calls(
-                        response.choices[0].message.tool_calls,
+        elif mode in [
+            MODE_CHAT, 
+            MODE_VISION, 
+            MODE_RESEARCH
+        ]:
+            if use_responses_api:
+                if response.output_text:
+                    output = response.output_text.strip()
+                if response.output:
+                    ctx.tool_calls = self.window.core.command.unpack_tool_calls_responses(
+                        response.output,
                     )
+            else:
+                if response.choices[0]:
+                    print(response.choices[0].message)
+                    if response.choices[0].message.content:
+                        output = response.choices[0].message.content.strip()
+                    elif response.choices[0].message.tool_calls:
+                        ctx.tool_calls = self.window.core.command.unpack_tool_calls(
+                            response.choices[0].message.tool_calls,
+                        )
         # audio
         elif mode in [MODE_AUDIO]:
             if response.choices[0]:
@@ -199,12 +253,42 @@ class Gpt:
                     ctx.audio_expires_ts = response.choices[0].message.audio.expires_at
                     ctx.is_audio = True
                     output = response.choices[0].message.audio.transcript  # from transcript
+                    self.chat.audio_prev_expires_ts = ctx.audio_expires_ts
+                    self.chat.audio_prev_id = ctx.audio_id
+                elif response.choices[0].message and not response.choices[0].message.audio:
+                    output = response.choices[0].message.content
+                    ctx.audio_id = self.chat.audio_prev_id
+                    ctx.audio_expires_ts = self.chat.audio_prev_expires_ts
+                if response.choices[0].message.tool_calls:
+                    ctx.tool_calls = self.window.core.command.unpack_tool_calls(
+                        response.choices[0].message.tool_calls,
+                    )
 
         ctx.set_output(output, ai_name)
-        ctx.set_tokens(
-            response.usage.prompt_tokens,
-            response.usage.completion_tokens,
-        )
+
+        if not use_responses_api:
+            ctx.set_tokens(
+                response.usage.prompt_tokens,
+                response.usage.completion_tokens,
+            )
+        else:
+            ctx.set_tokens(
+                response.usage.input_tokens,
+                response.usage.output_tokens,
+            )
+            if mode == MODE_CHAT:
+                # if image generation call in responses API
+                image_data = [
+                    output.result
+                    for output in response.output
+                    if output.type == "image_generation_call"
+                ]
+                if image_data:
+                    img_path = self.window.core.image.gen_unique_path(ctx)
+                    image_base64 = image_data[0]
+                    with open(img_path, "wb") as f:
+                        f.write(base64.b64decode(image_base64))
+                    ctx.images = [img_path]
         return True
 
     def quick_call(self, context: BridgeContext, extra: dict = None) -> str:
@@ -215,6 +299,7 @@ class Gpt:
         :param extra: Extra arguments
         :return: response content
         """
+        mode = context.mode
         prompt = context.prompt
         system_prompt = context.system_prompt
         max_tokens = context.max_tokens
@@ -223,7 +308,7 @@ class Gpt:
         if model is None:
             model = self.window.core.models.from_defaults()
 
-        client = self.get_client()
+        client = self.get_client(mode, model)
         messages = []
         messages.append({"role": "system", "content": system_prompt})
         messages.append({"role": "user", "content": prompt})
