@@ -6,9 +6,10 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.14 22:00:00                  #
+# Updated Date: 2025.06.30 18:00:00                  #
 # ================================================== #
 
+import base64
 import datetime
 import os
 from typing import Optional, Dict, Any
@@ -24,6 +25,10 @@ from pygpt_net.utils import trans
 
 
 class Image:
+
+    MODE_GENERATE = "generate"
+    MODE_EDIT = "edit"
+
     def __init__(self, window=None):
         """
         Image generation core
@@ -51,6 +56,12 @@ class Image:
         model = context.model
         num = extra.get("num", 1)
         inline = extra.get("inline", False)
+        sub_mode = self.MODE_GENERATE
+
+        # if attachments then switch mode to EDIT
+        attachments = context.attachments
+        if attachments and len(attachments) > 0:
+            sub_mode = self.MODE_EDIT
 
         if ctx is None:
             ctx = CtxItem()  # create empty context
@@ -65,6 +76,8 @@ class Image:
         self.worker.window = self.window
         self.worker.client = self.window.core.gpt.get_client()
         self.worker.ctx = ctx
+        self.worker.mode = sub_mode  # mode can be "generate" or "edit"
+        self.worker.attachments = attachments  # attachments for edit mode
         self.worker.raw = self.window.core.config.get('img_raw')
         self.worker.model = model.id  # model ID for generate image, e.g. "dall-e-3"
         self.worker.model_prompt = prompt_model  # model for generate prompt, not image!
@@ -121,9 +134,11 @@ class ImageWorker(QObject, QRunnable):
         self.client = None
         self.ctx = None
         self.raw = False
+        self.mode = Image.MODE_GENERATE  # default mode is generate
         self.model = "dall-e-3"
         self.quality = "standard"
         self.resolution = "1792x1024"
+        self.attachments = {}  # attachments for edit mode
         self.model_prompt = None
         self.input_prompt = None
         self.system_prompt = None
@@ -132,6 +147,7 @@ class ImageWorker(QObject, QRunnable):
         self.allowed_max_num = {
             "dall-e-2": 4,
             "dall-e-3": 1,
+            "gpt-image-1": 1,
         }
         self.allowed_resolutions = {
             "dall-e-2": [
@@ -144,6 +160,27 @@ class ImageWorker(QObject, QRunnable):
                 "1024x1792",
                 "1024x1024",
             ],
+            "gpt-image-1": [
+                "1536x1024",
+                "1024x1536",
+                "1024x1024",
+                "auto",
+            ],
+        }
+        self.allowed_quality = {
+            "dall-e-2": [
+                "standard",
+            ],
+            "dall-e-3": [
+                "standard",
+                "hd",
+            ],
+            "gpt-image-1": [
+                "auto",
+                "high",
+                "medium",
+                "low",
+            ],
         }
 
     @Slot()
@@ -188,22 +225,39 @@ class ImageWorker(QObject, QRunnable):
                 if resolution not in self.allowed_resolutions[self.model]:
                     resolution = self.allowed_resolutions[self.model][0]
 
+            quality = self.quality
+            if self.model in self.allowed_quality:
+                if quality not in self.allowed_quality[self.model]:
+                    quality = self.allowed_quality[self.model][0]
+
             # send to API
             response = None
-            if self.model == "dall-e-2":
-                response = self.client.images.generate(
+            if self.mode == Image.MODE_GENERATE:
+                if self.model == "dall-e-2":
+                    response = self.client.images.generate(
+                        model=self.model,
+                        prompt=self.input_prompt,
+                        n=self.num,
+                        size=resolution,
+                    )
+                elif self.model == "dall-e-3" or self.model == "gpt-image-1":
+                    response = self.client.images.generate(
+                        model=self.model,
+                        prompt=self.input_prompt,
+                        n=self.num,
+                        quality=quality,
+                        size=resolution,
+                    )
+            elif self.mode == Image.MODE_EDIT:
+                images = []
+                for uuid in self.attachments:
+                    attachment = self.attachments[uuid]
+                    if attachment.path and os.path.exists(attachment.path):
+                        images.append(open(attachment.path, "rb"))
+                response = self.client.images.edit(
                     model=self.model,
+                    image=images,
                     prompt=self.input_prompt,
-                    n=self.num,
-                    size=resolution,
-                )
-            elif self.model == "dall-e-3":
-                response = self.client.images.generate(
-                    model=self.model,
-                    prompt=self.input_prompt,
-                    n=self.num,
-                    quality=self.quality,
-                    size=resolution,
                 )
 
             # check response
@@ -215,20 +269,27 @@ class ImageWorker(QObject, QRunnable):
             for i in range(self.num):
                 if i >= len(response.data):
                     break
-                url = response.data[i].url
-                res = requests.get(url)
 
                 # generate filename
                 name = datetime.date.today().strftime(
                     "%Y-%m-%d") + "_" + datetime.datetime.now().strftime("%H-%M-%S") + "-" \
-                    + self.window.core.image.make_safe_filename(self.input_prompt) + "-" + str(i + 1) + ".png"
+                       + self.window.core.image.make_safe_filename(self.input_prompt) + "-" + str(i + 1) + ".png"
                 path = os.path.join(self.window.core.config.get_user_dir("img"), name)
 
                 msg = trans('img.status.downloading') + " (" + str(i + 1) + " / " + str(self.num) + ") -> " + str(path)
                 self.signals.status.emit(msg)
 
+                if response.data[i] is None:
+                    self.signals.error.emit("API Error: empty image data")
+                    return
+                if response.data[i].url:  # dall-e 2 and 3 returns URL
+                    res = requests.get(response.data[i].url)
+                    data = res.content
+                else:  # gpt-image-1 returns base64 encoded image
+                    data = base64.b64decode(response.data[i].b64_json)
+
                 # save image
-                if self.window.core.image.save_image(path, res.content):
+                if data and self.window.core.image.save_image(path, data):
                     paths.append(path)
                 else:
                     self.signals.error.emit("Error saving image")
