@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.01 01:00:00                  #
+# Updated Date: 2025.07.09 22:00:00                  #
 # ================================================== #
 
 from typing import Optional, List, Dict
@@ -93,7 +93,7 @@ class OpenAILLM(BaseLLM):
         :param stream: stream mode
         :return: LLM provider instance
         """
-        args = self.parse_args(model.llama_index)
+        args = self.parse_args(model.llama_index, window)
         if "model" not in args:
             args["model"] = model.id
 
@@ -141,7 +141,7 @@ class OpenAILLM(BaseLLM):
         :param stream: stream mode
         :return: LLM provider instance
         """
-        args = self.parse_args(model.llama_index)
+        args = self.parse_args(model.llama_index, window)
         if "model" not in args:
             args["model"] = model.id
         return LlamaOpenAIMultiModal(**args)
@@ -162,5 +162,5 @@ class OpenAILLM(BaseLLM):
         if config is not None:
             args = self.parse_args({
                 "args": config,
-            })
+            }, window)
         return OpenAIEmbedding(**args)
