@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.26 19:00:00                  #
+# Updated Date: 2025.06.28 16:00:00                  #
 # ================================================== #
 
 from PySide6.QtCore import Qt
@@ -23,6 +23,7 @@ from pygpt_net.core.types import (
     MODE_LANGCHAIN,
     MODE_LLAMA_INDEX,
     MODE_VISION,
+    MODE_RESEARCH,
 )
 from pygpt_net.ui.base.config_dialog import BaseConfigDialog
 from pygpt_net.ui.widget.dialog.editor import EditorDialog
@@ -102,12 +103,13 @@ class Preset(BaseConfigDialog):
         # modes
         mode_keys = [
             MODE_CHAT,
-            MODE_AUDIO,
+            MODE_LLAMA_INDEX,            
+            MODE_AUDIO,            
+            MODE_RESEARCH,
             MODE_COMPLETION,
             MODE_IMAGE,
             MODE_VISION,
-            MODE_LANGCHAIN,
-            MODE_LLAMA_INDEX,
+            # MODE_LANGCHAIN,
             MODE_AGENT_LLAMA,
             MODE_AGENT,
             MODE_EXPERT,
